<?php
/**
 *    This file is part of OXIDJson.
 *
 *    OXIDJson is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    OXIDJson is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this package.  If not, see <http://www.gnu.org/licenses/>.
 */

use Tonic\Response;

/**
 * @uri /oxlist/:class
 * @uri /oxlist/:class/:page
 * @uri /oxlist/:class/:page/:pageSize
 * @uri /oxlist/:class/:page/:pageSize/:orderBy
 * @uri /oxlist/:class/:propertyName/:comparator/:propertyValue/:page
 * @uri /oxlist/:class/:propertyName/:comparator/:propertyValue/:page/:pageSize
 * @uri /oxlist/:class/:propertyName/:comparator/:propertyValue/:page/:pageSize/:orderBy
 */
class OxRestList extends OxRestBase
{
    /**
     * @method GET
     * @json
     * @priority 2
     * @param string $class
     * @return Tonic\Response
     */
    public function getList($class) {

        // class must be oxarticlelist

        $class = $this->erx_get_class_alias($class);
        $this->erx_check_is_class_allowed($class);

        $class = ('oxarticle' == $class)? 'oxarticlelist' : $class;

        try {

            $ret = array();

            /** @var oxList $list */
            $list = oxNew($class);

            /** @var oxI18n $bo */
            $bo = $list->getBaseObject();

            $sSql = $this->erx_get_select_string($class, $bo->getViewName());

            if ($this->propertyName !== null && $this->propertyName !== '' && $this->propertyValue !== null && $this->propertyValue !== '') {

                $this->propertyName = preg_replace("/[^a-z0-9_]+/i", "", $this->propertyName);
                $this->propertyValue = mysql_real_escape_string($this->propertyValue);

                $sSql .= " WHERE {$this->propertyName} ";

                if ($this->comparator === "like") {
                    $sSql .= "LIKE '%{$this->propertyValue}%'";
                } elseif ($this->comparator === "gt") {
                    $sSql .= "> '{$this->propertyValue}'";
                } elseif ($this->comparator === "lt") {
                    $sSql .= "< '{$this->propertyValue}'";
                } else {
                    $sSql .= "= '{$this->propertyValue}'";
                }

                $ret['condition']  = $this->propertyName . ' ' . $this->comparator . ' ' . $this->propertyValue;
                $ret['comparator'] = $this->comparator;
            }

            $sSqlCount = "SELECT COUNT(*) AS cnt FROM (" . $sSql . ") AS t1";

            // order by

            $sOrderBy = "oxid ASC";

            if(isset($this->orderBy)) {
                $sOrderBy = mysql_real_escape_string(urldecode($this->orderBy));
            }

            $sSql .= " ORDER BY $sOrderBy";

            $iPerPage  = 50;

            if (isset($this->pageSize) && is_numeric($this->pageSize)) {
                $iPerPage = (int)$this->pageSize;
            }

            $cnt       = (int)oxDb::getDb()->getOne($sSqlCount);
            $iNumPages = $cnt > 0 ? (ceil($cnt / $iPerPage)) : 1;

            $iCurPage = 0;

            if (isset($this->page)) {
                $iCurPage = (int)$this->page;
            }

            $iLimitOffset = $iPerPage * $iCurPage;

            $sSql = $sSql . " LIMIT $iLimitOffset,$iPerPage";

            $l = array();

            $list->selectString(
                $sSql
            );

            foreach ($list->getArray() as $oxid => $oxObject) {

                $l[$oxid] = $this->_oxObject2Array($oxObject);

                // Inject deeplink into article

                if ('oxarticlelist' == $class) {

                    $l[$oxid]['deeplink'] = $oxObject->getBaseSeoLink(0,true);
                    $l[$oxid]['image'] = $oxObject->getMasterZoomPictureUrl(1);
                    $l[$oxid]['oxlongdesc'] = $oxObject->getLongDesc();
                    $l[$oxid]['originalprice'] = $l[$oxid]['oxprice'];
                    $l[$oxid]['oxprice'] = $oxObject->getPrice()->getPrice();

                    $sid_pos = strpos($l[$oxid]['deeplink'], '?force_sid=');

                    if ($sid_pos) {
                        $l[$oxid]['deeplink'] = substr($l[$oxid]['deeplink'], 0, $sid_pos);
                    }

                }

            }

            $ret['numPages']   = $iNumPages;
            $ret['curPage']    = $iCurPage;
            $ret['numObjects'] = $cnt;
            $ret['objectType'] = get_class($bo);
            $ret['result']     = $l;
            $ret['numCurr'] = count($l);

            return new Response(200, $ret);

        } catch (Exception $ex) {
            $this->_doLog("Error getting list: " . $ex->getMessage());
        }

        return new Response(404, "No data");

    }

}
