<?php
/**
 *    This file is part of OXIDJson.
 *
 *    OXIDJson is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    OXIDJson is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this package.  If not, see <http://www.gnu.org/licenses/>.
 */

use Tonic\Response;

/**
 * @uri /list/:class
 * @uri /list/:class/:page
 * @uri /list/:class/:page/:pageSize
 * @uri /list/:class/:page/:pageSize/:orderBy
 * @uri /list/:class/:propertyName/:comparator/:propertyValue/:page
 * @uri /list/:class/:propertyName/:comparator/:propertyValue/:page/:pageSize
 * @uri /list/:class/:propertyName/:comparator/:propertyValue/:page/:pageSize/:orderBy
 */
class OxRestListArray extends OxRestBase
{
    /**
     * @method GET
     * @json
     * @priority 2
     * @param string $class
     * @return Tonic\Response
     */
    public function arrayList($class)
    {

        $class = $this->erx_get_class_alias($class);
        $this->erx_check_is_class_allowed($class);

        try {
            $ret = array();
            $tableName = $this->_getTableName($class);
            $sSql = $this->erx_get_select_string($class, $tableName);

            if ($this->propertyName !== null && $this->propertyName !== '' && $this->propertyValue !== null && $this->propertyValue !== '') {

                $this->propertyName = preg_replace("/[^a-z0-9_]+/i", "", $this->propertyName);
                $this->propertyValue = mysql_real_escape_string($this->propertyValue);

                $sSql .= " WHERE {$this->propertyName} ";

                if ($this->comparator === "like") {
                    $sSql .= "LIKE '%{$this->propertyValue}%'";
                } elseif ($this->comparator === "gt") {
                    $sSql .= "> '{$this->propertyValue}'";
                } elseif ($this->comparator === "lt") {
                    $sSql .= "< '{$this->propertyValue}'";
                } else {
                    $sSql .= "= '{$this->propertyValue}'";
                }

                $ret['condition']  = $this->propertyName . ' ' . $this->comparator . ' ' . $this->propertyValue;
                $ret['comparator'] = $this->comparator;
            }

            $sSqlCount = "SELECT COUNT(*) AS cnt FROM (" . $sSql . ") AS t1";

            $sOrderBy = "oxid ASC";
            if(isset($this->orderBy)) {
                $sOrderBy = mysql_real_escape_string(urldecode($this->orderBy));
            }
            $sSql .= " ORDER BY $sOrderBy";

            $iPerPage  = 50;
            if (isset($this->pageSize) && is_numeric($this->pageSize)) {
                $iPerPage = (int)$this->pageSize;
            }
            $cnt       = (int)oxDb::getDb()->getOne($sSqlCount);
            $iNumPages = $cnt > 0 ? (ceil($cnt / $iPerPage)) : 1;

            $iCurPage = 0;
            if (isset($this->page)) {
                $iCurPage = (int)$this->page;
            }
            $iLimitOffset = $iPerPage * $iCurPage;

            $sSql = $sSql . " LIMIT $iLimitOffset,$iPerPage";

            //$this->_doLog($sSql);

            $l = array();
            // use only arrays

            $mode = defined('oxDb::FETCH_MODE_ASSOC') ? oxDb::FETCH_MODE_ASSOC : true;

            $a = oxDb::getDb($mode)->getAll($sSql);
            foreach ($a as $idx => $aData) {
                $aData2 = array();
                // lowercase keys from OXID to oxid etc. for the view
                foreach($aData as $aKey => $aVal) {
                    if (
                            strpos(strtolower($aKey), 'password') > 0 ||
                            strpos(strtolower($aKey), 'passsalt') > 0
                    ) {
                        $aVal = substr($aVal,0,3);
                    }
                    $aData2[strtolower($aKey)] = $aVal;

                }

                // Inject positions into order

                if ('oxorder' === $class) {

                    $sPositionsSql = 'SELECT * FROM oxorderarticles WHERE oxorderid = "' . $aData2['oxid'] . '"';

                    $aPositions = oxDb::getDb($mode)->getAll($sPositionsSql);
                    $aData2['positions'] = array();

                    foreach($aPositions as $aPositionKey => $aPosition) {

                        $aData2['positions'][strtolower($aPositionKey)] = array();

                        foreach($aPosition as $aKey => $aPosition) {
                            $aData2['positions'][strtolower($aPositionKey)][strtolower($aKey)] = $aPosition;
                        }

                    }

                }

                $l[$aData2['oxid']] = $aData2;

            }

            $ret['numPages']   = $iNumPages;
            $ret['curPage']    = $iCurPage;
            $ret['numObjects'] = $cnt;
            $ret['objectType'] = $class;
            $ret['result']     = $l;
            $ret['numCurr'] = count($l);
            //$this->_doLog("list: " . print_r($a, true));

            return new Response(200, $ret);
        } catch (Exception $ex) {
            $this->_doLog("Error getting list: " . $ex->getMessage());
        }

        return new Response(404, "No data");
    }


    /**
     * Get table name from object name
     * @param string $class
     * @return string
     */
    protected function _getTableName($class) {

        $class = $this->erx_get_class_alias($class);
        $this->erx_check_is_class_allowed($class);

        // check if the requested (oxList based) class exists, e.g. "oxarticleList"
        if(class_exists($class)) {
            /** @var oxList $list */
            $list = oxNew($class);
            /** @var oxI18n $bo */
            if(method_exists($list, "getBaseObject")) {
                $bo = $list->getBaseObject();
                return $bo->getViewName();
            }
        }
        // if not, assume we got a simple table name here, e.g. "oxarticles" or "oxorder"
        return $class;
    }
}
